@@ -69,7 +69,7 @@ for (cores in 1:min(2L, availableCores("multicore"))) {
     }
     mprintf(" - Resolving %d batchtools_multicore futures\n", length(x))
     if (globals || f$config$reg$cluster.functions$name == "Multicore") {
-      v <- unlist(values(x))
+      v <- unlist(value(x))
       stopifnot(all(v == 1:2))
     } else {
       v <- lapply(x, FUN = function(f) tryCatch(value(f), error = identity))
@@ -101,7 +101,7 @@ for (cores in 1:min(2L, availableCores("multicore"))) {
 ## CRAN processing times:
 ## On Windows 32-bit, don't run these tests
 if (fullTest || !isWin32) {
-  mprintf("*** batchtools_multicore() and errors\n", globals)
+  mprintf("*** batchtools_multicore() and errors\n")
   f <- batchtools_multicore({
     stop("Whoops!")
     1
